//RandomAgar v1.1
//(c) 2002 Gabriel Nivasch
#include "symm.h"
#include "RanMar.h"

//Thanks to Karel Suhajda for suggesting this random number generator.
//
//This random number generator is adapted from the code in the
//following web page:
//http://www.hh.se/stud/d98rolb/source/rand.html
//
//Reference:
//"Toward a Universal Random Number Generator" by George Marsaglia
//and Arif Zaman. Florida State University Report: FSU-SCRI-87-50
//(1987)
//
//This code can be useful in a lot of cases. Feel free to take it.

//This routine generates numbers between 0 and 2^24 - 1.
//To verify that it's working properly, use as seeds
//ij=1802 and kl=9373. Generate 20000 random numbers, then display
//the next 6. They should be:
//  6533892
// 14220222
//  7275067
//  6172232
//  8354498
// 10633180

RanMar:: RanMar()
{ init=false; }

void RanMar::genSeeds(int *s1, int *s2)
{
	//31329 and 30082 are relatively prime, so according to the
	//Chinese Remainder Theorem all combinations are possible.
	int t=time(NULL);
	*s1=t%31329;
	*s2=t%30082;
}

void RanMar::srand(int ij, int kl)
{
	//Ensure proper range:
	assert(ij>=0 && ij<31329 && kl>=0 && kl<30082);
	
	int i = ij/177 + 2,
	    j = ij%177 + 2,
	    k = kl/169 + 1,
	    l = kl%169;
	
	for (int ii=0; ii<97; ii++)
	{
		int s=0, t=1<<23;
		
		for (int jj=0; jj<24; jj++)
		{
			int m = (i*j*k) % 179;
			i=j;
			j=k;
			k=m;
			l = (53*l+1) % 169;
			if (((l*m) & (1<<5)) != 0)
				s|=t;
			t>>=1;
		}
		u[ii]=s;
	}
	
	c=362436;
	
	i97=96;
	j97=32;
	
	init=true;
} //srand()

int RanMar::rand()
{
	assert(init);
	
	int uni = u[i97] - u[j97--];
	if (uni<0) uni += 1<<24;
	
	u[i97--]=uni;
	if (i97<0) i97=96;
	if (j97<0) j97=96;
	
	c-=cd;
	if (c<0) c+=cm;
	
	uni-=c;
	if (uni<0) uni += 1<<24;
	
	return uni;
} //rand()