//RandomAgar v1.1
//(c) 2002 Gabriel Nivasch
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <set>
//Mac-specific:
#include <TextUtils.h>

#define assert(x) if(!(x)) { printf("Assert failed (%s %d).\n", \
	__FILE__, __LINE__); exit(1); }

//Function prototypes:
void setCreatorCode(const char *);

//The symmetries:
// (1)
class TorusSymm
{
	public:
		TorusSymm();
		TorusSymm(int, int, char *);
		static TorusSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 1; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const TorusSymm &);
		bool operator !=(const TorusSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x1, y1, x2, y2;
};

//(2)
class ClosedPillowcaseSymm
{
	public:
		ClosedPillowcaseSymm();
		ClosedPillowcaseSymm(int, int, char *);
		static ClosedPillowcaseSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 2; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const ClosedPillowcaseSymm &);
		bool operator !=(const ClosedPillowcaseSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x1, y1, x2, y2, ox, oy;
};

//(3)
class OrthoCylinderSymm
{
	public:
		OrthoCylinderSymm();
		OrthoCylinderSymm(int, int, char *);
		static OrthoCylinderSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 3; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoCylinderSymm &);
		bool operator !=(const OrthoCylinderSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis;
};

//(-3)
class DiagCylinderSymm
{
	public:
		DiagCylinderSymm();
		DiagCylinderSymm(int, int, char *);
		static DiagCylinderSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -3; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagCylinderSymm &);
		bool operator !=(const DiagCylinderSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, d1, d2, dsign;
};

//(4)
class OrthoKleinSymm
{
	public:
		OrthoKleinSymm();
		OrthoKleinSymm(int, int, char *);
		static OrthoKleinSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 4; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoKleinSymm &);
		bool operator !=(const OrthoKleinSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis;
};

//(-4)
class DiagKleinSymm
{
	public:
		DiagKleinSymm();
		DiagKleinSymm(int, int, char *);
		static DiagKleinSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -4; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagKleinSymm &);
		bool operator !=(const DiagKleinSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, d1, d2, dsign;
};

//(5)
class OrthoRectangleSymm
{
	public:
		OrthoRectangleSymm();
		OrthoRectangleSymm(int, int, char *);
		static OrthoRectangleSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 5; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoRectangleSymm &);
		bool operator !=(const OrthoRectangleSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis, yaxis;
};

//(-5)
class DiagRectangleSymm
{
	public:
		DiagRectangleSymm();
		DiagRectangleSymm(int, int, char *);
		static DiagRectangleSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -5; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagRectangleSymm &);
		bool operator !=(const DiagRectangleSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, d1, d2, daxis;
};

//(6)
class OrthoOpenPillowcaseSymm
{
	public:
		OrthoOpenPillowcaseSymm();
		OrthoOpenPillowcaseSymm(int, int, char *);
		static OrthoOpenPillowcaseSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 6; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoOpenPillowcaseSymm &);
		bool operator !=(const OrthoOpenPillowcaseSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis, yaxis;
};

//(-6)
class DiagOpenPillowcaseSymm
{
	public:
		DiagOpenPillowcaseSymm();
		DiagOpenPillowcaseSymm(int, int, char *);
		static DiagOpenPillowcaseSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -6; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagOpenPillowcaseSymm &);
		bool operator !=(const DiagOpenPillowcaseSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, d1, d2, dsign, daxis;
};

//(7)
class OrthoFootballSymm
{
	public:
		OrthoFootballSymm();
		OrthoFootballSymm(int, int, char *);
		static OrthoFootballSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 7; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoFootballSymm &);
		bool operator !=(const OrthoFootballSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis, yaxis;
};

//(-7)
class DiagFootballSymm
{
	public:
		DiagFootballSymm();
		DiagFootballSymm(int, int, char *);
		static DiagFootballSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -7; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagFootballSymm &);
		bool operator !=(const DiagFootballSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, d1, d2, daxis;
};

//(8)
class OrthoMobiusSymm
{
	public:
		OrthoMobiusSymm();
		OrthoMobiusSymm(int, int, char *);
		static OrthoMobiusSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 8; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoMobiusSymm &);
		bool operator !=(const OrthoMobiusSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis;
};

//(-8)
class DiagMobiusSymm
{
	public:
		DiagMobiusSymm();
		DiagMobiusSymm(int, int, char *);
		static DiagMobiusSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -8; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagMobiusSymm &);
		bool operator !=(const DiagMobiusSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, sign;
};

//(9)
class OrthoTurnoverHypoSymm
{
	public:
		OrthoTurnoverHypoSymm();
		OrthoTurnoverHypoSymm(int, int, char *);
		static OrthoTurnoverHypoSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 9; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoTurnoverHypoSymm &);
		bool operator !=(const OrthoTurnoverHypoSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, xaxis, yaxis;
};

//(-9)
class DiagTurnoverHypoSymm
{
	public:
		DiagTurnoverHypoSymm();
		DiagTurnoverHypoSymm(int, int, char *);
		static DiagTurnoverHypoSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -9; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagTurnoverHypoSymm &);
		bool operator !=(const DiagTurnoverHypoSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, axis;
};

//(10)
class TurnoverSymm
{
	public:
		TurnoverSymm();
		TurnoverSymm(int, int, char *);
		static TurnoverSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 10; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const TurnoverSymm &);
		bool operator !=(const TurnoverSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, x, y, center;
};

//(11)
class OrthoTriangleSymm
{
	public:
		OrthoTriangleSymm();
		OrthoTriangleSymm(int, int, char *);
		static OrthoTriangleSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 11; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoTriangleSymm &);
		bool operator !=(const OrthoTriangleSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, size, center;
};

//(-11)
class DiagTriangleSymm
{
	public:
		DiagTriangleSymm();
		DiagTriangleSymm(int, int, char *);
		static DiagTriangleSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -11; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagTriangleSymm &);
		bool operator !=(const DiagTriangleSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, size, center;
};

//(12)
class OrthoTurnoverLegSymm
{
	public:
		OrthoTurnoverLegSymm();
		OrthoTurnoverLegSymm(int, int, char *);
		static OrthoTurnoverLegSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return 12; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const OrthoTurnoverLegSymm &);
		bool operator !=(const OrthoTurnoverLegSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, size, center;
};

//(-12)
class DiagTurnoverLegSymm
{
	public:
		DiagTurnoverLegSymm();
		DiagTurnoverLegSymm(int, int, char *);
		static DiagTurnoverLegSymm getRandomSymm(int, int);
		void getcell(int, int, int *, int *);
		static int getID() { return -12; }
		int getxmax() { return xmax; }
		int getymax() { return ymax; }
		void getparams(char *);
		unsigned long long getHash();
		int getTileX();
		int getTileY();
		bool operator ==(const DiagTurnoverLegSymm &);
		bool operator !=(const DiagTurnoverLegSymm &s2) { return !(*this==s2); }
	private:
		int xmax, ymax, size, center;
};